import unittest

class TestGameTime(unittest.TestCase):
    def test_at_repeat(self):
        # game_time = GameTime()
        # self.assertEqual(expected, game_time.at_repeat())
        assert True # TODO: implement your test here

    def test_at_script_creation(self):
        # game_time = GameTime()
        # self.assertEqual(expected, game_time.at_script_creation())
        assert True # TODO: implement your test here

    def test_at_start(self):
        # game_time = GameTime()
        # self.assertEqual(expected, game_time.at_start())
        assert True # TODO: implement your test here

class TestSave(unittest.TestCase):
    def test_save(self):
        # self.assertEqual(expected, save())
        assert True # TODO: implement your test here

class TestRuntime(unittest.TestCase):
    def test_runtime(self):
        # self.assertEqual(expected, runtime(format))
        assert True # TODO: implement your test here

class TestUptime(unittest.TestCase):
    def test_uptime(self):
        # self.assertEqual(expected, uptime(format))
        assert True # TODO: implement your test here

class TestGametime(unittest.TestCase):
    def test_gametime(self):
        # self.assertEqual(expected, gametime(format))
        assert True # TODO: implement your test here

class TestGametimeToRealtime(unittest.TestCase):
    def test_gametime_to_realtime(self):
        # self.assertEqual(expected, gametime_to_realtime(secs, mins, hrs, days, weeks, months, yrs, format))
        assert True # TODO: implement your test here

class TestRealtimeToGametime(unittest.TestCase):
    def test_realtime_to_gametime(self):
        # self.assertEqual(expected, realtime_to_gametime(secs, mins, hrs, days, weeks, months, yrs, format))
        assert True # TODO: implement your test here

class TestInitGametime(unittest.TestCase):
    def test_init_gametime(self):
        # self.assertEqual(expected, init_gametime())
        assert True # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()
